#!/bin/sh

#########################################################################
# Plex Media Server init-script used to control start and stop.         #
# Maintainer: Plex Build Team                                           #
# Credit: Initial work and ongoing feedback by Dane22                   #
#########################################################################

# Location of QPKG conf file.
CONF=/etc/config/qpkg.conf

# Name of Plex Media Server install directory.
QPKG_NAME="PlexMediaServer"

# Grab Plex Media Server install directory, regardless of disk layout.
QPKG_DIR=$(getcfg -f $CONF $QPKG_NAME Install_path)

PID_FILE="${QPKG_DIR}/Library/Plex Media Server/plexmediaserver.pid"

case "$1" in
  start)

    # Is PMS enabled in QTS App Center?
    ENABLED=$(/sbin/getcfg $QPKG_NAME Enable -u -d FALSE -f $CONF)
    if [ "$ENABLED" != "TRUE" ]; then
        echo "$QPKG_NAME is disabled."
        exit 1
    fi

    # Is PMS already running?
    if [ -f "$PID_FILE" ]; then

      # Pid file was present, but sadly QNAP does a very brutal kill when shutting down.
      # Verify we only have a stray PID file (no active processes).
      ProcessCount=$(ps -ef | grep plex | wc -l)
      if [ $ProcessCount -gt 1 ]; then
        logMsg="Plex Media Server is currently running or hasn't been shutdown properly. Please stop it from QNAP AppCenter and start it again."
        echo $logMsg
        /sbin/log_tool -t 2 -a "$logMsg"
        exit 0
      fi
    fi

    # Obtain list and count of all installed GPUs.
    TEMPFILE=/tmp/PMS.GPU.$$.TMP
    gpuhal_app -l > $TEMPFILE
    GPU_COUNT=$(wc -l $TEMPFILE | awk '{print $1}')

    # Start with the first GPU we find.
    CURRENT_GPU=1

    # False until we find a supported, and properly configured GPU.
    SUPPORTED_GPU=0

    # Loop through the list of GPUs, starting with the first, until one first found -or- none found.
    while [ $CURRENT_GPU -le $GPU_COUNT ]
    do

      # Test this card, get the entire line
      CANDIDATE=$(head -$CURRENT_GPU $TEMPFILE | tail -1)

      # Test if NVIDIA card installed
      NVIDIA=$(echo $CANDIDATE | grep NVIDIA | wc -l)
      if [ $NVIDIA -gt 0 ]; then

        # Extract GPU_ID from candidate info, and get status.
        GPU_ID=$(echo $CANDIDATE | awk -F: '{print $1}')
        GPU_STATUS=$(gpuhal_app -s $GPU_ID)

        # Are the GPU Drivers installed?
        if [ $(echo $GPU_STATUS | grep driver_installed=1 | wc -l) -gt 0 ]; then

          # Is the GPU in QTS mode?
          if [ $(echo $GPU_STATUS | grep real_status=2 | wc -l) -gt 0 ]; then

            # Get the QPKG package name.
            GPU_QPKG=$(echo $GPU_STATUS | sed -e 's/^.*driverqpkg=//' | sed -e 's/,.*$//')

            # Get NVIDIA driver-env.sh information. (where all the variables are)
            GPU_SH=$(/sbin/getcfg $GPU_QPKG Install_Path -f /etc/config/qpkg.conf)/driver-env.sh

            # Translate the name to avoid collision and resolve absolute paths.
            DRIVER_ROOT="$(grep DRIVER_ROOT= $GPU_SH | awk -F= '{print $2}')"
            DRIVER_ROOT="$(echo $DRIVER_ROOT | sed -e s+\${QPKG_NAME}+$GPU_QPKG+)"

            GPU_LD_PATH=$(grep LD_LIBRARY_PATH= $GPU_SH | awk '{print $2}' | awk -F= '{print $2}' | sed -e 's/:$LD_LIBRARY_PATH//')
            GPU_LD_PATH="$(echo $GPU_LD_PATH | sed -e s+\${DRIVER_ROOT}+$DRIVER_ROOT+g)"

            # We have a viable GPU.
            SUPPORTED_GPU=1

          else

            # Log at INFO level that card is not in QTS mode an unable to be used by PMS.
            # User may not want PMS using it and/or user may already have a QSV-compatible CPU.
            log_tool -t 0 -a  "NVIDIA GPU detected and the drivers are installed.  Please switch card to 'QTS' mode for PMS to access it."

          fi
        else

          # GPU detected but no Driver is installed.
          log_tool -t 0 -a "NVIDIA GPU detected but the drivers are not installed.  Please install the NVIDIA drivers to use this card with Plex Media Server."

        fi
      fi

      # Do we have a viable GPU to use?
      if [ $SUPPORTED_GPU -eq 1 ]; then

        # Yes, we are done.
        break;
      fi

      # Default to an unsupported GPU, resetting flags along the way.
      SUPPORTED_GPU=0

      unset GPU_LD_PATH;  unset GPU_ID;  unset DRIVER_ROOT;
      unset QPKG_SH;      unset DRV_NAME

      # Set next candate.
      CURRENT_GPU=$(( $CURRENT_GPU + 1 ))

    done

    # Remove GPU temp file.
    rm -f $TEMPFILE

    logMsg="Starting Plex Media Server..."
    echo $logMsg

    /sbin/log_tool -t 0 -a "$logMsg"
    export PLEX_MEDIA_SERVER_MAX_PLUGIN_PROCS=6
    export LC_ALL=en_US.UTF-8
    export LANG=en_US.UTF-8
    export LANGUAGE=en_US.UTF-8
    export PLEX_MEDIA_SERVER_APPLICATION_SUPPORT_DIR="${QPKG_DIR}/Library"
    export TMPDIR="${QPKG_DIR}/Library/tmp"
    export LD_LIBRARY_PATH="${QPKG_DIR}/lib"
    export PLEX_MEDIA_SERVER_DEFAULT_PREFERENCES="HardwareAcceleratedCodecs=true&TranscoderCanOnlyRemuxVideo=false"

    # Set identification variables
    export PLEX_MEDIA_SERVER_INFO_VENDOR=QNAP
    export PLEX_MEDIA_SERVER_INFO_DEVICE="$(getsysinfo model)"
    export PLEX_MEDIA_SERVER_INFO_MODEL="$(uname -m)"
    export PLEX_MEDIA_SERVER_INFO_PLATFORM_VERSION="QTS $(getcfg system version).$(getcfg system Number)"

    # Append GPU Library path to LD_LIBRARY_PATH
    if [ "$GPU_LD_PATH" != "" ]; then
      export  LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${GPU_LD_PATH}
    fi

    # Wait a bit, since new firmware seems to cause a race condition.
    sleep 5
    ulimit -s 3000
    cd ${QPKG_DIR}
    ./Plex\ Media\ Server &
  ;;

  stop)
    if [ -f "$PID_FILE" ]; then
      logMsg="Stopping Plex Media Server..."
      echo $logMsg
      /sbin/log_tool -t 0 -a "$logMsg"
      kill $(cat "$PID_FILE")
      sleep 5

      # If pid file is a leftover from not shutting down properly, remove it now.
      if [ -f "$PID_FILE" ]; then
        rm "$PID_FILE"
      fi

    else
      logMsg="Plex Media Server hasn't been enabled or started..."
      echo $logMsg
      /sbin/log_tool -t 0 -a "$logMsg"
      exit 0
    fi
  ;;

  restart)
    $0 stop
    sleep 5
    $0 start
  ;;

  *)
    echo "Usage: $0 {start|stop|restart}"
    exit 1
esac

exit 0
